﻿using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Windows.Forms.Grid.Grouping;
using System.ComponentModel;
using Syncfusion.Grouping;

namespace Syncfusion.GridHelperClasses
{  
    /// <summary>
    /// Implements Office2007 filter.
    /// </summary>
    public class GridOffice2007Filter
    {
        /// <summary>
        /// Intializes a new <see cref="GridOffice2007Filter"/>
        /// </summary>
        public GridOffice2007Filter()
            : base()
        {
        }

        # region ShowOffice2007FilterOnMouseHover
        private static bool showOffice2007FilterOnMouseHover = true;
        /// <summary>
        /// set this property to enable the Office2007 filter on mouse over.if false is set, the filter displays directly when the grid has been wired.
        /// </summary>
        [DefaultValue("true")]
        public static bool ShowOffice2007FilterOnMouseHover
        {
            get
            {
                return showOffice2007FilterOnMouseHover;
            }
            set
            {
                if (showOffice2007FilterOnMouseHover != value)
                {
                    showOffice2007FilterOnMouseHover = value;
                }
            }
        }
        # endregion

        #region EnableFilteredColumnIcon
        private static bool enableFilteredColumnIcon = false;
        /// <summary>
        /// Gets or Sets this property to stick the filter icon in the filtered column. 
        /// This property is effective only if the ShowOffice2007FilterOnMouseHover property is in true.
        /// </summary>
        [DefaultValue(false)]
        public static bool EnableFilteredColumnIcon
        {
            get
            {
                return enableFilteredColumnIcon;
            }
            set
            {
                if (enableFilteredColumnIcon != value)
                {
                    enableFilteredColumnIcon = value;
                }
            }
        }
        #endregion

        /// <summary>
        /// Hook the grouping grid to the office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void WireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {                
                groupingGrid.BeginUpdate();
                groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new Grid2007ExcelFilterCellModel(groupingGrid.TableModel);            
                groupingGrid.TableDescriptor.IsExcelFilterWired = true;
                groupingGrid.TableModel.UpdateColumnWidths(true);
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
            }
        }

        /// <summary>
        /// Unhook the grouping grid from the office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void UnWireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {
                groupingGrid.BeginUpdate();
                groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new GridTableColumnHeaderCellModel(groupingGrid.TableModel);
                groupingGrid.TableDescriptor.IsExcelFilterWired = false;
                groupingGrid.TableModel.UpdateColumnWidths(true);
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
                groupingGrid = null;
            }
        }
    }

    /// <summary>
    /// Implements optimized Office2007 filter.
    /// </summary>
    public class GridExcelFilter
    {
        /// <summary>
        /// Intializes a new <see cref="GridExcelFilter"/>
        /// </summary>
        public GridExcelFilter()
            : base()
        {
        }

        /// <summary>
        /// Hook the grouping grid to the optimized office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void WireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {
                groupingGrid.BeginUpdate();
                if (!AllowIndividualColumnWiring)
                    groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new GridExcelFilterCellModel(groupingGrid.TableModel);
                else
                    groupingGrid.TableModel.CellModels.Add("GridExcelFilterCell", new GridExcelFilterCellModel(groupingGrid.TableModel));
                RemoveStandardFilterLists(groupingGrid);
                groupingGrid.TableDescriptor.IsExcelFilterWired = true;
                groupingGrid.TableModel.UpdateColumnWidths(true);
                groupingGrid.QueryFilterBarChoices += new GridQueryFilterBarChoicesEventHandler(gridGroupingControl1_QueryFilterBarChoices);
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
            }
        }
        public Dictionary<string, List<string>> GetFilterCollection(GridGroupingControl grid)
        {
            GridExcelFilterCellRenderer renderer = grid.TableControl.CellRenderers["ColumnHeaderCell"] as GridExcelFilterCellRenderer;
            if (renderer != null)
                return renderer.filterCollection;
            else
                return null;
        }
        private static bool enableFilteredColumnIcon = false;
        /// <summary>
        /// Gets or Sets this property to stick the filter icon in the filtered column. 
        /// </summary>
        [DefaultValue(false)]
        public static bool EnableFilteredColumnIcon
        {
            get
            {
                return enableFilteredColumnIcon;
            }
            set
            {
                if (enableFilteredColumnIcon != value)
                {
                    enableFilteredColumnIcon = value;
                }
            }
        }
        bool allowIndividualColumnWiring = false;
        /// <summary>
        /// It allows to set the desired filter on specified column,if the value is set to True, 
        /// through which the filter can be set in column using 'this.gridGroupingControl1.TableDescriptor.Columns[ColumnName].Appearance.ColumnHeaderCell.CellType = "GridExcelFilterCell"'
        /// </summary>
        [DefaultValue(false)]
        public bool AllowIndividualColumnWiring
        {
            get
            {
                return allowIndividualColumnWiring;
            }
            set
            {
                allowIndividualColumnWiring = value;
            }
        }
        /// <summary>
        /// An internal method to remove the choices.
        /// </summary>
        /// <param name="gridGroupingControl1">The grouping grid.</param>
        private void RemoveStandardFilterLists(GridGroupingControl gridGroupingControl1)
        {
            List<SummaryDescriptor> toRemove = new List<SummaryDescriptor>();
            foreach (SummaryDescriptor sd in gridGroupingControl1.TableDescriptor.Summaries)
            {
                if (sd.Name.EndsWith("FilterBarChoices"))
                {
                    toRemove.Add(sd);
                }
            }

            foreach (SummaryDescriptor sd in toRemove)
            {
                gridGroupingControl1.TableDescriptor.Summaries.Remove(sd);
            }
        }

        /// <summary>
        /// Clears all the filter choices of the GridGroupingControl.
        /// </summary>
        /// <param name="grid">GridGroupingControl</param>
        public void ClearFilters(GridGroupingControl grid)
        {
            GridExcelFilterCellRenderer renderer = grid.TableControl.CellRenderers["ColumnHeaderCell"] as GridExcelFilterCellRenderer;
            if (renderer != null)
            {
                renderer.ClearFilters();
                grid.Table.TableDirty = true;
                grid.Refresh();
            }
        }

        /// <summary>
        /// This event is used to provide list of choices.
        /// </summary>
        /// <param name="sender">Sender as GridGroupingControl</param>
        /// <param name="e">event data.</param>
        void gridGroupingControl1_QueryFilterBarChoices(object sender, GridQueryFilterBarChoicesEventArgs e)
        {
            List<object> choicesOrdered = new List<object>();
            List<object> choices = new List<object>();

            GridGroupingControl ggc = sender as GridGroupingControl;

            if (ggc != null)
            {
                string tableName = e.Column.TableDescriptor.Name;


                if (tableName != ggc.TableDescriptor.Name)
                    return;

                GridTable table = ggc.GetTable(tableName);

                string format = (e.Column.Appearance.AnyRecordFieldCell.Format != null && e.Column.Appearance.AnyRecordFieldCell.Format.Length > 0)
                    ? string.Format("{{0:{0}}}", e.Column.Appearance.AnyRecordFieldCell.Format) : "{0}";

                foreach (Record o in table.FilteredRecords)
                {
                    string val = string.Format(format, o.GetValue(e.Column.MappingName));// pdc[e.Column.MappingName].GetValue(o.GetData()));
                    int loc = choicesOrdered.BinarySearch(val);
                    if (loc < 0)
                    {
                        choicesOrdered.Insert(-loc - 1, val);
                        choices.Add(val);
                    }
                }
            }

            e.UniqueFilterBarValues = choices.ToArray();
        }

        /// <summary>
        /// Unhook the grouping grid from the optimized office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void UnWireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {
                groupingGrid.BeginUpdate();
                groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new GridTableColumnHeaderCellModel(groupingGrid.TableModel);
                groupingGrid.QueryFilterBarChoices -= new GridQueryFilterBarChoicesEventHandler(gridGroupingControl1_QueryFilterBarChoices);
                groupingGrid.TableDescriptor.IsExcelFilterWired = false;
                groupingGrid.TableModel.UpdateColumnWidths(true);
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
                groupingGrid = null;
            }
        }
    }
}
